#include "bsp.h"
#include "spi.h"
#include "tty.h"

static void __attribute__((constructor))
spi_initialize ()
{
  ssmr2.b = 0;

  sser.te = 0;     /* transmitter disabled */
  sser.re = 0;     /* receiver disabled */
  ssmr2.ssums = 1; /* 4-wire mode */
  ssmr.cphs = 0;   /* change data at odd edge */
  ssmr.cpos = 1;   /* clock polarity - L when idle */
  ssmr.mls = 0;    /* msb first */
  sscrh.mss = 1;   /* operate as master */
  ssmr2.scks = 1;  /* SCK is clock */
  ssmr2.soos = 0;  /* SSO pin is CMOS */

  sscrh.cks = 6;   /* f1/4 */
  //sscrh.cks = 0;   /* f1/256 */

  sscrh.rsstp = 0; /* multi-byte receive enabled */
  sssr.orer = 0;   /* no overrun errors */
  sser.re = 1;     /* transmitter enabled */
  sser.te = 1;     /* receiver enabled */
  sser.rie = 0;    /* no receive interrupts */
  sser.teie = 0;   /* no transmit end interrupt */
  sser.tie = 0;    /* no transmit interrupt */

  prcr.prc2 = 1;
  pd0.b = 0xff;
  pd2.b = 0xff;
  pd3.b0 = 1;
  pd3.b1 = 1;
  p3.b = ~0;
}

void
spi_reset (void)
{
  p3.b0 = 0;
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  p3.b0 = 1;
  wait_ms (5);
}

#undef spi_read

unsigned int
spi_write (unsigned char val)
{
  sstdr = val;
  cprintf("\033[31m%b\033[0m", val);
  while (sssr.rdrf == 0);
  return ssrdr;
}

unsigned int
spi_read ()
{
  unsigned char val;
  sstdr = 0;
  while (sssr.rdrf == 0);
  val = ssrdr;
  cprintf("\033[32m%b\033[0m", val);
  return val;
}
